WM__PATTERNS__ = ["safegraph_weekly_patterns.*"] # , "safegraph_monthly_patterns.*"]
DATASET = ["safegraph_core", "safegraph_geometry", "safegraph_monthly_patterns", "safegraph_weekly_patterns",]
INNER_DATASET = ["safegraph_core.*", "safegraph_geometry.*", "safegraph_monthly_patterns.*", "safegraph_weekly_patterns.*",] # for dataset column functionality
__PATTERNS__ = {
    "safegraph_core": { 
        "__header__": "safegraph_core {\n",
        # "placekey": "\t\tplacekey\n",
        "parent_placekey": "\t\tparent_placekey\n",
        "location_name": "\t\tlocation_name\n",
        "safegraph_brand_ids": "\t\tsafegraph_brand_ids\n",
        "brands": """\t\tbrands """,
        "top_category": "\t\ttop_category\n",
        "sub_category": "\t\tsub_category\n",
        "naics_code": "\t\tnaics_code\n",
        "latitude": "\t\tlatitude\n",
        "longitude": "\t\tlongitude\n",
        "street_address": "\t\tstreet_address\n",
        "city": "\t\tcity\n",
        "region": "\t\tregion\n",
        "postal_code": "\t\tpostal_code\n",
        "iso_country_code": "\t\tiso_country_code\n",
        "phone_number": "\t\tphone_number\n",
        "open_hours": "\t\topen_hours\n",
        "category_tags": "\t\tcategory_tags\n",
        "opened_on": "\t\topened_on\n",
        "closed_on": "\t\tclosed_on\n",
        "tracking_closed_since": "\t\ttracking_closed_since\n",
        "geometry_type": "\t\tgeometry_type\n",
        "__footer__": "\t\t}"
    },
    "safegraph_geometry": {
        "__header__": "safegraph_geometry {\n",
        # "placekey": "\t\tplacekey\n",
        "parent_placekey": "\t\tparent_placekey\n",
        "location_name": "\t\tlocation_name\n",
        "safegraph_brand_ids": "\t\tsafegraph_brand_ids\n",
        "brands": """\t\tbrands """,
        "latitude": "\t\tlatitude\n",
        "longitude": "\t\tlongitude\n",
        "street_address": "\t\tstreet_address\n",
        "city": "\t\tcity\n",
        "region": "\t\tregion\n",
        "postal_code": "\t\tpostal_code\n",
        "iso_country_code": "\t\tiso_country_code\n",
        "polygon_wkt": "\t\tpolygon_wkt\n",
        "polygon_class": "\t\tpolygon_class\n",
        "includes_parking_lot": "\t\tincludes_parking_lot\n",
        "is_synthetic": "\t\tis_synthetic\n",
        "enclosed": "\t\tenclosed\n",
        "__footer__": "\t\t}"
    },
    "safegraph_monthly_patterns": {
        "__header__": "safegraph_monthly_patterns {\n",#_DATE_BLOCK_ {\n",
        # "placekey": "\t\tplacekey\n",
        "parent_placekey": "\t\tparent_placekey\n",
        "location_name": "\t\tlocation_name\n",
        "street_address": "\t\tstreet_address\n",
        "city": "\t\tcity\n",
        "region": "\t\tregion\n",
        "postal_code": "\t\tpostal_code\n",
        "safegraph_brand_ids": "\t\tsafegraph_brand_ids\n",
        "brands": """\t\tbrands """,
        "date_range_start": "\t\tdate_range_start\n",
        "date_range_end": "\t\tdate_range_end\n",
        "raw_visit_counts": "\t\traw_visit_counts\n",
        "raw_visitor_counts": "\t\traw_visitor_counts\n",
        "visits_by_day": "\t\tvisits_by_day\n",
        "poi_cbg": "\t\tpoi_cbg\n",
        "visitor_home_cbgs": "\t\tvisitor_home_cbgs\n",
        "visitor_home_aggregation": "\t\tvisitor_home_aggregation\n",
        "visitor_daytime_cbgs": "\t\tvisitor_daytime_cbgs\n",
        "visitor_country_of_origin": "\t\tvisitor_country_of_origin\n",
        "distance_from_home": "\t\tdistance_from_home\n",
        "median_dwell": "\t\tmedian_dwell\n",
        "bucketed_dwell_times": "\t\tbucketed_dwell_times\n",
        "related_same_day_brand": "\t\trelated_same_day_brand\n",
        "related_same_month_brand": "\t\trelated_same_month_brand\n",
        "popularity_by_hour": "\t\tpopularity_by_hour\n",
        "popularity_by_day": "\t\tpopularity_by_day\n",
        "device_type": "\t\tdevice_type\n",
        "__footer__": "\t\t}"
    },
    "safegraph_weekly_patterns": {
        "__header__": "safegraph_weekly_patterns _DATE_BLOCK_ {\n",
        # "placekey": "\t\tplacekey\n",
        "parent_placekey": "\t\tparent_placekey\n",
        "location_name": "\t\tlocation_name\n",
        "street_address": "\t\tstreet_address\n",
        "city": "\t\tcity\n",
        "region": "\t\tregion\n",
        "postal_code": "\t\tpostal_code\n",
        "iso_country_code": "\t\tiso_country_code\n",
        "safegraph_brand_ids": "\t\tsafegraph_brand_ids\n",
        "brands": "\t\tbrands\n",
        "date_range_start": "\t\tdate_range_start\n",
        "date_range_end": "\t\tdate_range_end\n",
        "raw_visit_counts": "\t\traw_visit_counts\n",
        "raw_visitor_counts": "\t\traw_visitor_counts\n",
        "visits_by_day": "\t\tvisits_by_day\n",
        "visits_by_each_hour": "\t\tvisits_by_each_hour\n",
        "poi_cbg": "\t\tpoi_cbg\n",
        "visitor_home_cbgs": "\t\tvisitor_home_cbgs\n",
        "visitor_home_aggregation": "\t\tvisitor_home_aggregation\n",
        "visitor_daytime_cbgs": "\t\tvisitor_daytime_cbgs\n",
        "visitor_country_of_origin": "\t\tvisitor_country_of_origin\n",
        "distance_from_home": "\t\tdistance_from_home\n",
        "median_dwell": "\t\tmedian_dwell\n",
        "bucketed_dwell_times": "\t\tbucketed_dwell_times\n",
        "related_same_day_brand": "\t\trelated_same_day_brand\n",
        "related_same_week_brand": "\t\trelated_same_week_brand\n",
        "device_type": "\t\tdevice_type\n",
        "__footer__": "\t\t}"
    },
}
__VALUE_TYPES__ = {
    "naics_code": int,
    "latitude": float,
    "longitude": float,
}


from safegraphql import client
import pandas as pd
import random
from safegraphql.types import __PATTERNS__

df_type = type(pd.DataFrame())
try:
    apiKey = open("apiKey.txt").readlines()[0]
except Exception as e:
    print("create file apiKey.txt and put your api key from https://shop.safegraph.com/api inside")
    raise e
    

sgql_client = client.HTTP_Client(apiKey)


arr = []
banned_keys = ["__header__", "__footer__"]
keys = [i for i in __PATTERNS__.keys()] 
for i in keys:
    arr += [i for i in __PATTERNS__[i] if i not in banned_keys]

placekeys = [
        "224-222@5vg-7gv-d7q", # "222-222@5qw-shj-7qz", "222-222@5s6-pyc-7qz", "zzy-222@5xc-k8q-zmk",
        "zzw-222@8fy-fjg-b8v", # (Disney World)
        "zzy-227@5sb-8cw-pjv", # (O'Hare Airport)
        "222-223@65y-rxx-djv", # (Walmart in Albany, NY)
        ] 

def test_combine_list():
    date = {"date_range_start": "2021-07-10", "date_range_end": "2021-08-01"}
    core = sgql_client.lookup(placekeys=placekeys, product="core", columns=["placekey", "brands", "naics_code"], return_type="list", date=date)
    sgql_client.save(path="core.csv")
    geometry = sgql_client.lookup(placekeys=placekeys, product="geometry", columns=["placekey", "location_name", "street_address", "city", "enclosed"], 
        return_type="list", date=date)
    sgql_client.save(path="geometry.csv")
    #monthly_patterns = sgql_client.lookup(placekeys=placekeys, product="monthly_patterns", columns=["placekey", "date_range_start", "date_range_end","raw_visit_counts", "raw_visitor_counts"], return_type="pandas")
    monthly_patterns = sgql_client.lookup(placekeys=placekeys, product="monthly_patterns", columns=["placekey", "date_range_start", "date_range_end","raw_visit_counts", "raw_visitor_counts"], 
        return_type="list", date=date)
    sgql_client.save(path="monthly_patterns.csv")
    arr_ = [ core, geometry, monthly_patterns ]
    inner_df = sgql_client.sg_merge(arr_, how="inner")
    sgql_client.save(path="inner_df.csv")
    outer_df = sgql_client.sg_merge(arr_, how="outer")
    sgql_client.save(path="outer_df.csv")
    columns = []
    for i in arr_:
        for j in i:
            columns += [k for k in j if k not in columns]
    assert len(columns) == len([i for i in inner_df[0]])
    assert len(columns) == len([i for i in outer_df[0]])
    # weekly pattterns error case
    weekly_patterns = sgql_client.lookup(placekeys=placekeys, product="weekly_patterns", columns=["placekey", "brands", "visitor_home_cbgs", "distance_from_home", "related_same_day_brand"], return_type="list", date=date)
    arr_ = [ core, geometry, monthly_patterns, weekly_patterns ]
    try:
        inner_df = sgql_client.sg_merge(arr_, how="inner")
        # outer_df = sgql_client.sg_merge(arr_, how="outer")
    except Exception as e:
        assert type(e) == TypeError

def test_combine_pandas():
    date = ["2021-07-10", "2021-08-01", "2020-05-11"]
    core = sgql_client.lookup(placekeys=placekeys, product="core", columns=["placekey", "brands", "naics_code"], 
        return_type="pandas", date=date)
    sgql_client.save(path="core.csv")
    geometry = sgql_client.lookup(placekeys=placekeys, product="geometry", columns=["placekey", "location_name", "street_address", "city", "enclosed"], 
        return_type="pandas", date=date)
    sgql_client.save(path="geometry.csv")
    #monthly_patterns = sgql_client.lookup(placekeys=placekeys, product="monthly_patterns", columns=["placekey", "date_range_start", "date_range_end","raw_visit_counts", "raw_visitor_counts"], return_type="pandas")
    monthly_patterns = sgql_client.lookup(placekeys=placekeys, product="monthly_patterns", columns=["placekey", "date_range_start", "date_range_end","raw_visit_counts", "raw_visitor_counts"], 
        return_type="pandas", date=date)
    sgql_client.save(path="monthly_patterns.csv")
    arr_ = [ core, geometry, monthly_patterns ]
    inner_df = sgql_client.sg_merge(arr_, how="inner")
    sgql_client.save(path="inner_df.csv")
    outer_df = sgql_client.sg_merge(arr_, how="outer")
    sgql_client.save(path="outer_df.csv")
    columns = []
    for i in arr_:
        columns += [i for i in i.columns if i not in columns]
    assert len(columns) == inner_df.shape[1]
    assert len(columns) == outer_df.shape[1]
    # weekly pattterns error case
    weekly_patterns = sgql_client.lookup(placekeys=placekeys, product="weekly_patterns", columns=["placekey", "brands", "visitor_home_cbgs", "distance_from_home", "related_same_day_brand"], 
        return_type="pandas", date=date)
    arr_ = [ core, geometry, monthly_patterns, weekly_patterns ]
    try:
        inner_df = sgql_client.sg_merge(arr_, how="inner")
        # outer_df = sgql_client.sg_merge(arr_, how="outer")
    except Exception as e:
        assert type(e) == TypeError

def test_get_place_by_locatian_name_address():
    date = ["2018-06-05", "2019-06-05", "2020-06-05", "2021-06-05"]
    # safegraph_core
    assert type(sgql_client.lookup_by_name(date=date, 
        product="core",
        location_name= "Taco Bell", 
        street_address= "710 3rd St", 
        city= "San Francisco", 
        region= "CA", 
        iso_country_code= "US",
        return_type="pandas",
        columns="*")) == df_type
    # safegraph_geometry
    assert type(sgql_client.lookup_by_name(date=date, 
        product="geometry",
        location_name= "Taco Bell", 
        street_address= "710 3rd St", 
        city= "San Francisco", 
        region= "CA", 
        iso_country_code= "US",
        return_type="list",
        columns="*")) == list
    # safegraph_weekly_patterns
    weekly = sgql_client.lookup_by_name(date=date, product="weekly_patterns", location_name= "Taco Bell", street_address= "710 3rd St", city= "San Francisco", region= "CA", iso_country_code= "US", return_type="pandas", columns=["related_same_week_brand", "related_same_day_brand", "bucketed_dwell_times"])
    type(weekly) == df_type

def test_search(): 
    date = ["2021-05-05", "2021-06-05", "2021-04-05"]
    # safegraph_core
    assert type(sgql_client.search( product="core", columns="*", brand = "starbucks", brand_id = None, naics_code = None, phone_number = None, street_address = None, city = None, region = None, postal_code = None, iso_country_code = None, 
        max_results=70, after_result_number=10, return_type="pandas", date=date)) == df_type
    # safegraph_geometry
    assert type(sgql_client.search( product="geometry", brand = "starbucks", columns=["location_name", "street_address", "city", "enclosed", "latitude", "iso_country_code"], brand_id = None, naics_code = None, phone_number = None, street_address = None, city = None, region = None, postal_code = None, iso_country_code = None, 
        max_results=55, after_result_number=15,  return_type="list", date=date)) == list
    # safegraph_weekly_patterns
    naics_code = "445120"
    assert type(sgql_client.search(product="weekly_patterns", columns =["median_dwell", "city", "brands", "visitor_home_cbgs", "distance_from_home", "related_same_day_brand"], naics_code = naics_code, date=date)) == df_type

    # safegraph_monthly_patterns
    try:
        city = 'fsahfsadhfsadkjfsadjf'
        sgql_client.search(product="monthly_patterns", columns = "*", city = city, date=date)
    except Exception as e:
        assert type(e) == client.safeGraphError

def test_lookup():  
    date = {"date_range_start": "2021-07-10", "date_range_end": "2021-08-01"}
    # safegraph_core
    assert type(sgql_client.lookup(placekeys=placekeys, product="core", columns="*", return_type="pandas", date=date)) == df_type
    core = sgql_client.lookup(placekeys=placekeys, product="core", columns=["placekey", "brands", "naics_code"], return_type="pandas", date=date)
    assert type(core) == df_type
    # safegraph_geometry
    assert type(sgql_client.lookup(placekeys=placekeys, product="geometry", columns="*", return_type="pandas", date=date)) == df_type
    geometry = sgql_client.lookup(placekeys=placekeys, product="geometry", columns=["location_name", "street_address", "city", "enclosed"], return_type="pandas", date=date)
    assert type(geometry) == df_type
    # safegraph_weekly_patterns
    assert type(sgql_client.lookup(placekeys=placekeys, product="weekly_patterns", columns="*", return_type="pandas", date=date)) == df_type
    weekly_patterns = sgql_client.lookup(placekeys=placekeys, product="weekly_patterns", columns=["brands", "visitor_home_cbgs", "distance_from_home", "related_same_day_brand"], return_type="pandas", date=date)
    assert type(weekly_patterns) == df_type
    # safegraph_monthly_patterns
    assert type(sgql_client.lookup(placekeys=placekeys, product="monthly_patterns", columns="*", return_type="pandas", date=date)) == df_type
    monthly_patterns = sgql_client.lookup(placekeys=placekeys, product="monthly_patterns", columns=["date_range_start", "date_range_end","raw_visit_counts", "raw_visitor_counts"], return_type="pandas", date=date)
    assert type(monthly_patterns) == df_type

    try:
        sgql_client.lookup(placekeys=placekeys, 
        product="core",
        columns=
            ["fakes", "fake2"] + 
            random.sample(arr,random.randint(1, len(arr))), 
        return_type="pandas", date=date)
    except Exception as e:
        assert type(e) == client.safeGraphError

def test_non_weekly_date_req():
    # weekly monthly
    # safegraph_core                
    date = {"date_range_start": "2021-07-10", "date_range_end": "2021-08-01"}                                                                                                                                                                                                                                                   
    assert type(sgql_client.lookup(placekeys=placekeys, product="weekly_patterns", columns="*", return_type="list", date=date)) == list                                                                                                                                                      
    core = sgql_client.lookup(placekeys=placekeys, product="weekly_patterns", columns="*", return_type="pandas", date=date)                                                                                                                                     
    assert type(core) == df_type                              
    # monthly_pattern                                                                                                                                                                                                                         
    assert type(sgql_client.lookup(placekeys=placekeys, product="monthly_patterns", columns="*", return_type="list", date=date)) == list                                                                                                                                                      
    core = sgql_client.lookup(placekeys=placekeys, product="monthly_patterns", columns="*", return_type="pandas", date=date)                                                                                                                                     
    assert type(core) == df_type     

    # NON weekly monthly
    # safegraph_core
    assert type(sgql_client.lookup(placekeys=placekeys, product="core", columns="*", return_type="pandas", date=date)) == df_type
    core = sgql_client.lookup(placekeys=placekeys, product="core", columns=["placekey", "brands", "naics_code"], return_type="pandas", date=date)
    assert type(core) == df_type
    # safegraph_geometry
    assert type(sgql_client.lookup(placekeys=placekeys, product="geometry", columns="*", return_type="pandas", date=date)) == df_type
    geometry = sgql_client.lookup(placekeys=placekeys, product="geometry", columns=["location_name", "street_address", "city", "enclosed"], return_type="pandas", date=date)
    assert type(geometry) == df_type
    # safegraph_weekly_patterns
    assert type(sgql_client.lookup(placekeys=placekeys, product="weekly_patterns", columns="*", return_type="pandas", date=date)) == df_type
    weekly_patterns = sgql_client.lookup(placekeys=placekeys, product="weekly_patterns", columns=["brands", "visitor_home_cbgs", "distance_from_home", "related_same_day_brand"], return_type="pandas", date=date)
    assert type(weekly_patterns) == df_type
    # safegraph_monthly_patterns
    assert type(sgql_client.lookup(placekeys=placekeys, product="monthly_patterns", columns="*", return_type="pandas", date=date)) == df_type
    monthly_patterns = sgql_client.lookup(placekeys=placekeys, product="monthly_patterns", columns=["date_range_start", "date_range_end","raw_visit_counts", "raw_visitor_counts"], return_type="pandas", date=date)
    assert type(monthly_patterns) == df_type



# def test_safegraph_weekly_patterns():
#     sgql_client.date = ["2021-08-05", "2021-08-12", "2021-08-19"]
#     sgql_client.patterns_version = "weekly"
#     df = sgql_client.lookup(placekeys, product="core", columns=["safegraph_brand_ids", "date_range_start", "visits_by_day"], return_type="pandas") 
#     assert type(df) == df_type
#     df = sgql_client.lookup(placekeys, product="geometry", columns="safegraph_weekly_patterns.*", return_type="pandas")
#     assert type(df) == df_type
#     df = sgql_client.lookup(placekeys, product="weekly_patterns", columns="*", date="2021-01-01", patterns_version="weekly", return_type="pandas")
#     assert type(df) == df_type
#     columns = ['related_same_month_brand', 'visits_by_each_hour', "location_name", "longitude", "date_range_start"]
#     try:
#         df = sgql_client.lookup(placekeys, columns=columns, return_type="pandas")
#     except Exception as e:
#         assert type(e) == ValueError


# def test_search_pagination():
#     city = "Philadelphia"
#     brand = "Starbucks"

#     brand_search_initial = sgql_client.search(columns = ['placekey'], brand = brand, city = city, max_results = 10, after_result_number = 0)
#     brand_search_pagination1 = sgql_client.search(columns = ['placekey'], brand = brand, city = city, max_results = 25, after_result_number = 0)
#     brand_search_pagination2 = sgql_client.search(columns = ['placekey'], brand = brand, city = city, max_results = 25, after_result_number = 10)
#     assert (set(brand_search_initial.placekey) == set(brand_search_pagination1.placekey).difference(brand_search_pagination2.placekey))

# def test_overlap():
#     city = "Philadelphia"
#     brand = "Starbucks"

#     brand_search_initial = sgql_client.search(columns = ['placekey'], brand = brand, city = city, max_results = 10, after_result_number = 0)
#     brand_search_pagination1 = sgql_client.search(columns = ['placekey'], brand = brand, city = city, max_results = 10, after_result_number = 10)
#     for i in range(len(brand_search_initial)):
#         assert brand_search_initial.loc[i].values[0] != brand_search_pagination1.loc[i].values[0]

# def test_null_cases():
#     null_check = [
#         "placekey",
#         "location_name",
#         "top_category",
#         "sub_category",
#         "naics_code",
#         "latitude",
#         "longitude",
#         "street_address",
#         "city",
#         "region",
#         "postal_code",
#         "iso_country_code",
#         # "tracking_closed_since",
#         # "geometry_type",
#         # "polygon_wkt",
#         # "polygon_class",
#         # "is_synthetic",
#         # "enclosed",
#         # "date_range_start",
#         # "date_range_end",
#         # "raw_visit_counts",
#         # "raw_visitor_counts",
#         # "visits_by_day",
#         # "visitor_home_cbgs",
#         # "visitor_home_aggregation",
#         # "visitor_daytime_cbgs",
#         # "visitor_country_of_origin",
#         # "distance_from_home",
#         # "median_dwell",
#         # "bucketed_dwell_times",
#         # "related_same_day_brand",
#         # "related_same_month_brand",
#         # "popularity_by_hour",
#         # "popularity_by_day",
#         # "device_type",
#     ] 
#     df = sgql_client.lookup(placekeys, columns="*", return_type="pandas")
#     for i in null_check:
#         assert(df[i].isnull().values.any() == False)
#     for i in range(len(df)):
#         # Check that the naics_code column is a string in any Pandas dataframe results.
#         assert(type(df.loc[i]["naics_code"]) == str)

# def test_save():
#     # Read in the result of save() and make sure it matches the original dataframe.
#     df = sgql_client.lookup(placekeys, columns="*", return_type="pandas")
#     path = "results.csv"
#     sgql_client.save(path)
#     saved_df = pd.read_csv(path)
#     # if same shape means same values
#     assert(df.shape == saved_df.shape)



from setuptools import find_packages, setup
import os

DIRECTORY = os.path.dirname(__file__)

#REQUIREMENTS = open(os.path.join(DIRECTORY, "REQUIREMENTS.txt")).read().split()
VERSION = open(os.path.join(DIRECTORY, "safegraphql", "__version__.py")).read()
READ_ME = open(os.path.join(DIRECTORY, "README.md")).read()

setup(
    name='safegraphQL',
    version=VERSION,
    description='graphQL API of safegraph.com using Python functions',
    long_description=READ_ME,
    long_description_content_type="text/markdown",
    url="https://github.com/echong-SG/API-python-client-MKilic",
    author="Renas Mirkan Kilic",
    author_email="mirkanbaba1@gmail.com",
    license='MIT',
    classifiers=[
        "Intended Audience :: Science/Research",
        "Topic :: Software Development :: Libraries",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    keywords="api graphql safegraph gql safegraph-api",
    packages=find_packages(include=['safegraphql']),
    install_requires=[
        # "graphql-core>=2.3.2",
        # "yarl>=1.6,<2.0",
        "gql>=2.0.0",
        "pandas>=1.3.2"
    ],
    setup_requires=['pytest-runner'],
    tests_require=['pytest==4.4.1'],
    test_suite='tests',
    platforms="any",
)
